import { Worker } from "worker_threads";
import os from "os";

export default function solver(challenges, config) {
  const totalChallenges = challenges.length;
  const numWorkers =
    config?.workerCount || Math.min(totalChallenges, 14, os.cpus().length);

  let challengesProcessed = 0;
  let nextChallengeIndex = 0;
  let activeWorkers = 0;
  const results = [];

  return new Promise((resolve) => {
    function startWorker() {
      if (nextChallengeIndex < totalChallenges && activeWorkers < numWorkers) {
        const currentChallengeIndex = nextChallengeIndex;
        const [salt, target] = challenges[currentChallengeIndex];
        nextChallengeIndex++;
        activeWorkers++;

        const worker = new Worker("./worker.js", {
          workerData: { salt, target, challengeIndex: currentChallengeIndex },
        });

        worker.on("message", (result) => {
          results.push([result.salt, result.target, result.nonce]);
          challengesProcessed++;
          activeWorkers--;

          if (challengesProcessed === totalChallenges) {
            resolve(results);
          } else {
            startWorker();
          }
        });
      }
    }

    for (let i = 0; i < numWorkers; i++) {
      startWorker();
    }

    if (totalChallenges === 0) {
      resolve([]);
    }
  });
}
